#!/usr/local/bin/ruby
# Tiny Tiny eRuby --- String output 
# 	Copyright (c) 1999-2000 Masatoshi SEKI 
#       You can redistribute it and/or modify it under the same term as Ruby.
# 	$Id: strio.rb,v 1.4 2000/04/23 15:09:10 mas Exp $
# 	$Author: mas $

require 'thread'

module DefOut
  def write(s)
    out = @writers.fetch(Thread.current, @default)
    out.write(s)
  end

  def set(obj)
    @m.synchronize do
      @writers[Thread.current] = obj
    end
  end

  def cleanup
    @m.synchronize do
      @writers.delete_if do |k, v|
	not k.alive?
      end
    end
  end
  module_function :write, :set, :cleanup

  @m = Mutex.new
  @writers = Hash.new
  @default = $>
  $> = DefOut
  @th = Thread.new {
    while true
      sleep 60
      $>.cleanup
    end
  }

=begin
  trace_var('$>') do 
    DefOut.set($>)
    $> = DefOut
  end
=end
end

class StringIO
  def initialize(str='')
    @str = str.to_s
  end
  attr :str

  def write(str)
    @str.concat(str)
  end

  def as_stdout(safe=nil)
    t = Thread.new {
      out = StringIO.new
      DefOut.set(out)
      $SAFE = safe if safe
      yield
      out.str
    }
    DefOut.cleanup
    t.value
  end
  module_function :as_stdout
end


